@@ -6,60 +6,32 @@
 
 #include "dbgdriver.h"
 #include "exprwnd.h"
-#include <qstringlist.h>
+#include <QStringList>
 #include <ctype.h>
+#include <signal.h>
 #include <stdlib.h>			/* strtol, atoi */
 #include <algorithm>
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "mydebug.h"
 #include <assert.h>
 
 
 DebuggerDriver::DebuggerDriver() :
-	KProcess(),
 	m_state(DSidle),
-	m_output(0),
-	m_outputLen(0),
-	m_outputAlloc(0),
-	m_activeCmd(0),
-	m_promptMinLen(0),
-	m_promptLastChar('\0')
+	m_activeCmd(0)
 {
-    m_outputAlloc = 4000;
-    m_output = new char[m_outputAlloc];
-
-    m_prompt[0] = '\0';
-    // Derived classes can set either m_prompt or m_promptRE.
-    // If m_promptRE is set, it must include the '$' at the end.
-    // m_promptLastChar and m_promptMinLen must also be set.
-
     // debugger process
-    connect(this, SIGNAL(receivedStdout(KProcess*,char*,int)),
-	    SLOT(slotReceiveOutput(KProcess*,char*,int)));
-    connect(this, SIGNAL(wroteStdin(KProcess*)), SLOT(slotCommandRead(KProcess*)));
-    connect(this, SIGNAL(processExited(KProcess*)), SLOT(slotExited(KProcess*)));
+    connect(this, SIGNAL(readyReadStandardOutput()), SLOT(slotReceiveOutput()));
+    connect(this, SIGNAL(bytesWritten(qint64)), SLOT(slotCommandRead()));
+    connect(this, SIGNAL(finished(int, QProcess::ExitStatus)),
+	    SLOT(slotExited()));
 }
 
 DebuggerDriver::~DebuggerDriver()
 {
-    delete[] m_output;
     flushHiPriQueue();
     flushLoPriQueue();
 }
 
-int DebuggerDriver::commSetupDoneC()
-{
-    TRACE(__PRETTY_FUNCTION__);
-
-    if (!KProcess::commSetupDoneC())
-	return 0;
-
-    close(STDERR_FILENO);
-    return dup2(STDOUT_FILENO, STDERR_FILENO) != -1;
-}
-
 
 bool DebuggerDriver::startup(QString cmdStr)
 {
@@ -74,46 +46,36 @@ bool DebuggerDriver::startup(QString cmdStr)
     if (cmdStr.isEmpty())
 	cmdStr = defaultInvocation();
 
-    QStringList cmd = QStringList::split(' ', cmdStr);
-    clearArguments();
-    for (QStringList::iterator i = cmd.begin(); i != cmd.end(); ++i) {
-	*this << *i;
-    }
+    QStringList cmd = cmdStr.split(' ', QString::SkipEmptyParts);
+    if (cmd.isEmpty())
+	return false;
+    QString pgm = cmd.takeFirst();
 
-    if (!start(KProcess::NotifyOnExit,
-	       KProcess::Communication(KProcess::Stdin|KProcess::Stdout))) {
+    setProcessChannelMode(MergedChannels);
+    start(pgm, cmd);
+    if (!waitForStarted(-1))
 	return false;
-    }
 
     // open log file
     if (!m_logFile.isOpen() && !m_logFileName.isEmpty()) {
-	m_logFile.setName(m_logFileName);
-	m_logFile.open(IO_WriteOnly);
+	m_logFile.setFileName(m_logFileName);
+	m_logFile.open(QIODevice::WriteOnly);
     }
 
-    // these must be set by derived classes in their constructor
-    assert(m_promptMinLen > 0);		// _must_ have a prompt
-    assert(m_promptLastChar != '\0');	// last char _must_ be fixed
-    assert(m_prompt[0] == '\0' || strlen(m_prompt) == m_promptMinLen);
-    // either a prompt or a regexp
-    assert(m_prompt[0] != '\0' && m_promptMinLen < sizeof(m_prompt) ||
-	   !m_promptRE.isEmpty() && m_promptRE.isValid());
-
     return true;
 }
 
-void DebuggerDriver::slotExited(KProcess*)
+void DebuggerDriver::slotExited()
 {
     static const char txt[] = "\n====== debugger exited ======\n";
     if (m_logFile.isOpen()) {
-	m_logFile.writeBlock(txt,sizeof(txt)-1);
+	m_logFile.write(txt,sizeof(txt)-1);
     }
 
     // reset state
     m_state = DSidle;
     // empty buffer
-    m_outputLen = 0;
-    *m_output = '\0';
+    m_output.clear();
 }
 
 
@@ -128,7 +90,7 @@ CmdQueueItem* DebuggerDriver::executeCmdString(DbgCommand cmd,
 	if (m_state == DSrunningLow) {
 	    // take the liberty to interrupt the running command
 	    m_state = DSinterrupted;
-	    kill(SIGINT);
+	    ::kill(pid(), SIGINT);
 	    ASSERT(m_activeCmd != 0);
 	    TRACE(QString().sprintf("interrupted the command %d",
 		  (m_activeCmd ? m_activeCmd->m_cmd : -1)));
@@ -220,12 +182,20 @@ void DebuggerDriver::writeCommand()
     m_activeCmd = cmd;
     TRACE("in writeCommand: " + cmd->m_cmdString);
 
-    const char* str = cmd->m_cmdString;
-    writeStdin(const_cast<char*>(str), cmd->m_cmdString.length());
+    QByteArray str = cmd->m_cmdString.toLocal8Bit();
+    const char* data = str.data();
+    qint64 len = str.length();
+    while (len > 0) {
+	qint64 n = write(data, len);
+	if (n <= 0)
+	    break;	// ignore error
+	len -= n;
+	data += n;
+    }
 
     // write also to log file
     if (m_logFile.isOpen()) {
-	m_logFile.writeBlock(str, cmd->m_cmdString.length());
+	m_logFile.write(str, cmd->m_cmdString.length());
 	m_logFile.flush();
     }
 
@@ -256,7 +226,7 @@ void DebuggerDriver::flushCommands(bool hipriOnly)
     }
 }
 
-void DebuggerDriver::slotCommandRead(KProcess*)
+void DebuggerDriver::slotCommandRead()
 {
     TRACE(__PRETTY_FUNCTION__);
 
@@ -280,36 +250,43 @@ void DebuggerDriver::slotCommandRead(KProcess*)
 	break;
     }
 
-    // re-receive delayed output
+    // process delayed output
     while (!m_delayedOutput.empty()) {
 	QByteArray delayed = m_delayedOutput.front();
 	m_delayedOutput.pop();
-	slotReceiveOutput(0, const_cast<char*>(delayed.data()), delayed.size());
+	processOutput(delayed);
     }
 }
 
-void DebuggerDriver::slotReceiveOutput(KProcess*, char* buffer, int buflen)
+void DebuggerDriver::slotReceiveOutput()
 {
+    QByteArray data = readAllStandardOutput();
+
     /*
      * The debugger should be running (processing a command) at this point.
      * If it is not, it is still idle because we haven't received the
-     * wroteStdin signal yet, in which case there must be an active command
+     * bytesWritten signal yet, in which case there must be an active command
      * which is not commited.
      */
     if (m_state == DScommandSent || m_state == DScommandSentLow) {
 	ASSERT(m_activeCmd != 0);
 	ASSERT(!m_activeCmd->m_committed);
 	/*
-	 * We received output before we got signal wroteStdin. Collect this
-	 * output, it will be re-sent by commandRead when it gets the
+	 * We received output before we got signal bytesWritten. Collect this
+	 * output, it will be processed by commandRead when it gets the
 	 * acknowledgment for the uncommitted command.
 	 */
-	m_delayedOutput.push(QByteArray().duplicate(buffer, buflen));
+	m_delayedOutput.push(data);
 	return;
     }
+    processOutput(data);
+}
+
+void DebuggerDriver::processOutput(const QByteArray& data)
+{
     // write to log file (do not log delayed output - it would appear twice)
     if (m_logFile.isOpen()) {
-	m_logFile.writeBlock(buffer, buflen);
+	m_logFile.write(data);
 	m_logFile.flush();
     }
     
@@ -321,7 +298,7 @@ void DebuggerDriver::slotReceiveOutput(KProcess*, char* buffer, int buflen)
      */
     if (m_activeCmd == 0 && m_state != DSinterrupted) {
 	// ignore the output
-	TRACE("ignoring stray output: " + QString::fromLatin1(buffer, buflen));
+	TRACE("ignoring stray output: " + QString(data));
 	return;
     }
     ASSERT(m_state == DSrunning || m_state == DSrunningLow || m_state == DSinterrupted);
@@ -330,66 +307,16 @@ void DebuggerDriver::slotReceiveOutput(KProcess*, char* buffer, int buflen)
     // collect output until next prompt string is found
     
     // accumulate it
-    if (m_outputLen + buflen >= m_outputAlloc) {
-	/*
-	 * Must enlarge m_output: double it. Note: That particular
-	 * sequence of commandes here ensures exception safety.
-	 */
-	int newSize = m_outputAlloc * 2;
-	char* newBuf = new char[newSize];
-	memcpy(newBuf, m_output, m_outputLen);
-	delete[] m_output;
-	m_output = newBuf;
-	m_outputAlloc = newSize;
-    }
-    memcpy(m_output+m_outputLen, buffer, buflen);
-    m_outputLen += buflen;
-    m_output[m_outputLen] = '\0';
+    m_output += data;
 
-    /*
-     * If there's a prompt string in the collected output, it must be at
-     * the very end. In order to quickly find out whether there is a prompt
-     * string, we check whether the last character of m_output is identical
-     * to the last character of the prompt string. Only if it is, we check
-     * for the full prompt string.
-     * 
-     * Note: Using the regular expression here is most expensive, because
-     * it translates m_output to a QString each time.
-     * 
-     * Note: It could nevertheless happen that a character sequence that is
-     * equal to the prompt string appears at the end of the output,
-     * although it is very, very unlikely (namely as part of a string that
-     * lingered in gdb's output buffer due to some timing/heavy load
-     * conditions for a very long time such that that buffer overflowed
-     * exactly at the end of the prompt string look-a-like).
-     */
-    int promptStart = -1;
-    if (m_output[m_outputLen-1] == m_promptLastChar &&
-	m_outputLen >= m_promptMinLen)
-    {
-	// this is a candidate for a prompt at the end,
-	// now see if there really is
-	if (m_prompt[0] != '\0') {
-	    if (strncmp(m_output+m_outputLen-m_promptMinLen,
-			m_prompt, m_promptMinLen) == 0)
-	    {
-		promptStart = m_outputLen-m_promptMinLen;
-	    }
-	} else {
-	    QString output = QString::fromLatin1(m_output, m_outputLen);
-#if QT_VERSION >= 300
-	    promptStart = m_promptRE.search(output);
-#else
-	    promptStart = m_promptRE.match(output);
-#endif
-	}
-    }
+    // check for a prompt
+    int promptStart = findPrompt(m_output);
     if (promptStart >= 0)
     {
 	// found prompt!
 
 	// terminate output before the prompt
-	m_output[promptStart] = '\0';
+	m_output.resize(promptStart);
 
 	/*
 	 * We've got output for the active command. But if it was
@@ -409,8 +336,7 @@ void DebuggerDriver::slotReceiveOutput(KProcess*, char* buffer, int buflen)
 	}
 
 	// empty buffer
-	m_outputLen = 0;
-	*m_output = '\0';
+	m_output.clear();
 	// also clear delayed output if interrupted
 	if (m_state == DSinterrupted) {
 	    m_delayedOutput = std::queue<QByteArray>();
