@@ -10,28 +10,26 @@
 #include "typetable.h"
 #include "exprwnd.h"
 #include "pgmsettings.h"
-#include "programconfig.h"
-#include <qregexp.h>
-#include <qfileinfo.h>
-#include <qlistbox.h>
-#include <qstringlist.h>
-#include <kapplication.h>
+#include <QFileInfo>
+#include <QListWidget>
+#include <QApplication>
+#include <kcodecs.h>			// KMD5
 #include <kconfig.h>
 #include <klocale.h>			/* i18n */
 #include <kmessagebox.h>
+#include <kstandarddirs.h>
 #include <ctype.h>
 #include <stdlib.h>			/* strtol, atoi */
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>			/* sleep(3) */
-#endif
+#include <algorithm>
 #include "mydebug.h"
 
 
 KDebugger::KDebugger(QWidget* parent,
 		     ExprWnd* localVars,
 		     ExprWnd* watchVars,
-		     QListBox* backtrace) :
-	QObject(parent, "debugger"),
+		     QListWidget* backtrace) :
+	QObject(parent),
 	m_ttyLevel(ttyFull),
 	m_memoryFormat(MDTword | MDThex),
 	m_haveExecutable(false),
@@ -45,18 +43,16 @@ KDebugger::KDebugger(QWidget* parent,
 	m_watchVariables(*watchVars),
 	m_btWindow(*backtrace)
 {
-    m_envVars.setAutoDelete(true);
-
-    connect(&m_localVariables, SIGNAL(expanded(QListViewItem*)),
-	    SLOT(slotExpanding(QListViewItem*)));
-    connect(&m_watchVariables, SIGNAL(expanded(QListViewItem*)),
-	    SLOT(slotExpanding(QListViewItem*)));
+    connect(&m_localVariables, SIGNAL(itemExpanded(QTreeWidgetItem*)),
+	    SLOT(slotExpanding(QTreeWidgetItem*)));
+    connect(&m_watchVariables, SIGNAL(itemExpanded(QTreeWidgetItem*)),
+	    SLOT(slotExpanding(QTreeWidgetItem*)));
     connect(&m_localVariables, SIGNAL(editValueCommitted(VarTree*, const QString&)),
 	    SLOT(slotValueEdited(VarTree*, const QString&)));
     connect(&m_watchVariables, SIGNAL(editValueCommitted(VarTree*, const QString&)),
 	    SLOT(slotValueEdited(VarTree*, const QString&)));
 
-    connect(&m_btWindow, SIGNAL(highlighted(int)), SLOT(gotoFrame(int)));
+    connect(&m_btWindow, SIGNAL(currentRowChanged(int)), this, SLOT(gotoFrame(int)));
 
     emit updateUI();
 }
@@ -95,7 +91,7 @@ bool KDebugger::debugProgram(const QString& name,
 {
     if (m_d != 0 && m_d->isRunning())
     {
-	QApplication::setOverrideCursor(waitCursor);
+	QApplication::setOverrideCursor(Qt::WaitCursor);
 
 	stopDriver();
 
@@ -113,10 +109,11 @@ bool KDebugger::debugProgram(const QString& name,
     // wire up the driver
     connect(driver, SIGNAL(activateFileLine(const QString&,int,const DbgAddr&)),
 	    this, SIGNAL(activateFileLine(const QString&,int,const DbgAddr&)));
-    connect(driver, SIGNAL(processExited(KProcess*)), SLOT(gdbExited(KProcess*)));
+    connect(driver, SIGNAL(finished(int, QProcess::ExitStatus)),
+	    SLOT(gdbExited()));
     connect(driver, SIGNAL(commandReceived(CmdQueueItem*,const char*)),
 	    SLOT(parse(CmdQueueItem*,const char*)));
-    connect(driver, SIGNAL(wroteStdin(KProcess*)), SIGNAL(updateUI()));
+    connect(driver, SIGNAL(bytesWritten(qint64)), SIGNAL(updateUI()));
     connect(driver, SIGNAL(inferiorRunning()), SLOT(slotInferiorRunning()));
     connect(driver, SIGNAL(enterIdleState()), SLOT(backgroundUpdate()));
     connect(driver, SIGNAL(enterIdleState()), SIGNAL(updateUI()));
@@ -130,10 +127,10 @@ bool KDebugger::debugProgram(const QString& name,
 
     // get debugger command from per-program settings
     if (m_programConfig != 0) {
-	m_programConfig->setGroup(GeneralGroup);
-	m_debuggerCmd = readDebuggerCmd();
+	KConfigGroup g = m_programConfig->group(GeneralGroup);
+	m_debuggerCmd = readDebuggerCmd(g);
 	// get terminal emulation level
-	m_ttyLevel = TTYLevel(m_programConfig->readNumEntry(TTYLevelEntry, ttyFull));
+	m_ttyLevel = TTYLevel(g.readEntry(TTYLevelEntry, int(ttyFull)));
     }
     // the rest is read in later in the handler of DCexecutable
 
@@ -286,12 +283,8 @@ bool KDebugger::runUntil(const QString& fileName, int lineNo)
 {
     if (isReady() && m_programActive && !m_programRunning) {
 	// strip off directory part of file name
-	QString file = fileName;
-	int offset = file.findRev("/");
-	if (offset >= 0) {
-	    file.remove(0, offset+1);
-	}
-	m_d->executeCmd(DCuntil, file, lineNo, true);
+	QFileInfo fi(fileName);
+	m_d->executeCmd(DCuntil, fi.fileName(), lineNo, true);
 	m_programRunning = true;
 	return true;
     } else {
@@ -406,11 +399,7 @@ CmdQueueItem* KDebugger::executeBreakpoint(const Breakpoint* bp, bool queueOnly)
     else if (bp->address.isEmpty())
     {
 	// strip off directory part of file name
-	QString file = bp->fileName;
-	int offset = file.findRev("/");
-	if (offset >= 0) {
-	    file.remove(0, offset+1);
-	}
+	QString file = QFileInfo(bp->fileName).fileName();
 	if (queueOnly) {
 	    cmd = m_d->queueCmd(bp->temporary ? DCtbreakline : DCbreakline,
 				file, bp->lineNo, DebuggerDriver::QMoverride);
@@ -432,6 +421,17 @@ CmdQueueItem* KDebugger::executeBreakpoint(const Breakpoint* bp, bool queueOnly)
     return cmd;
 }
 
+bool KDebugger::infoLine(QString file, int lineNo, const DbgAddr& addr)
+{
+    if (isReady() && !m_programRunning) {
+	CmdQueueItem* cmd = m_d->executeCmd(DCinfoline, file, lineNo);
+	cmd->m_addr = addr;
+	return true;
+    } else {
+	return false;
+    }
+}
+
 bool KDebugger::enableDisableBreakpoint(QString file, int lineNo,
 					const DbgAddr& address)
 {
@@ -614,17 +614,17 @@ void KDebugger::stopDriver()
      * this timeout be reached, the only reasonable thing one could do then
      * is exiting kdbg.
      */
-    kapp->processEvents(1000);		/* ideally, this will already shut it down */
+    QApplication::processEvents(QEventLoop::AllEvents, 1000);
     int maxTime = 20;			/* about 20 seconds */
     while (m_haveExecutable && maxTime > 0) {
 	// give gdb time to die (and send a SIGCLD)
 	::sleep(1);
 	--maxTime;
-	kapp->processEvents(1000);
+	QApplication::processEvents(QEventLoop::AllEvents, 1000);
     }
 }
 
-void KDebugger::gdbExited(KProcess*)
+void KDebugger::gdbExited()
 {
     /*
      * Save settings, but only if gdb has already processed "info line
@@ -670,11 +670,16 @@ void KDebugger::gdbExited(KProcess*)
 QString KDebugger::getConfigForExe(const QString& name)
 {
     QFileInfo fi(name);
-    QString pgmConfigFile = fi.dirPath(true);
-    if (!pgmConfigFile.isEmpty()) {
-	pgmConfigFile += '/';
-    }
-    pgmConfigFile += ".kdbgrc." + fi.fileName();
+    QString dir = fi.absolutePath();
+
+    // The session file for the given executable consists of
+    // a hash of the directory, followed by the program name.
+    // Assume that the first 15 positions of the hash are unique;
+    // this keeps the file names short.
+    QString hash = KMD5(dir.toUtf8()).base64Digest();
+    hash.replace('/', QString());	// avoid directory separators
+    QString pgmConfigFile = hash.left(15) + "-" + fi.fileName();
+    pgmConfigFile = KStandardDirs::locateLocal("sessions", pgmConfigFile);
     TRACE("program config file = " + pgmConfigFile);
     return pgmConfigFile;
 }
@@ -685,7 +690,13 @@ void KDebugger::openProgramConfig(const QString& name)
 
     QString pgmConfigFile = getConfigForExe(name);
 
-    m_programConfig = new ProgramConfig(pgmConfigFile);
+    m_programConfig = new KConfig(pgmConfigFile);
+
+    // this leaves a clue behind in the config file which
+    // executable it applies to; it is mostly intended for
+    // users peeking into the file
+    KConfigGroup g = m_programConfig->group(GeneralGroup);
+    g.writeEntry("ExecutableFile", name);
 }
 
 const char EnvironmentGroup[] = "Environment";
@@ -701,30 +712,30 @@ const char ExprFmt[] = "Expr%d";
 void KDebugger::saveProgramSettings()
 {
     ASSERT(m_programConfig != 0);
-    m_programConfig->setGroup(GeneralGroup);
-    m_programConfig->writeEntry(FileVersion, 1);
-    m_programConfig->writeEntry(ProgramArgs, m_programArgs);
-    m_programConfig->writeEntry(WorkingDirectory, m_programWD);
-    m_programConfig->writeEntry(OptionsSelected, m_boolOptions);
-    m_programConfig->writeEntry(DebuggerCmdStr, m_debuggerCmd);
-    m_programConfig->writeEntry(TTYLevelEntry, int(m_ttyLevel));
+    KConfigGroup gg = m_programConfig->group(GeneralGroup);
+    gg.writeEntry(FileVersion, 1);
+    gg.writeEntry(ProgramArgs, m_programArgs);
+    gg.writeEntry(WorkingDirectory, m_programWD);
+    gg.writeEntry(OptionsSelected, m_boolOptions.toList());
+    gg.writeEntry(DebuggerCmdStr, m_debuggerCmd);
+    gg.writeEntry(TTYLevelEntry, int(m_ttyLevel));
     QString driverName;
     if (m_d != 0)
 	driverName = m_d->driverName();
-    m_programConfig->writeEntry(DriverNameEntry, driverName);
+    gg.writeEntry(DriverNameEntry, driverName);
 
     // write environment variables
     m_programConfig->deleteGroup(EnvironmentGroup);
-    m_programConfig->setGroup(EnvironmentGroup);
-    QDictIterator<EnvVar> it = m_envVars;
-    EnvVar* var;
+    KConfigGroup eg = m_programConfig->group(EnvironmentGroup);
     QString varName;
     QString varValue;
-    for (int i = 0; (var = it) != 0; ++it, ++i) {
+    int i = 0;
+    for (std::map<QString,QString>::iterator it = m_envVars.begin(); it != m_envVars.end(); ++it, ++i)
+    {
 	varName.sprintf(Variable, i);
 	varValue.sprintf(Value, i);
-	m_programConfig->writeEntry(varName, it.currentKey());
-	m_programConfig->writeEntry(varValue, var->value);
+	eg.writeEntry(varName, it->first);
+	eg.writeEntry(varValue, it->second);
     }
 
     saveBreakpoints(m_programConfig);
@@ -733,12 +744,11 @@ void KDebugger::saveProgramSettings()
     // first get rid of whatever was in this group
     m_programConfig->deleteGroup(WatchGroup);
     // then start a new group
-    m_programConfig->setGroup(WatchGroup);
-    VarTree* item = m_watchVariables.firstChild();
+    KConfigGroup wg = m_programConfig->group(WatchGroup);
     int watchNum = 0;
-    for (; item != 0; item = item->nextSibling(), ++watchNum) {
-	varName.sprintf(ExprFmt, watchNum);
-	m_programConfig->writeEntry(varName, item->getText());
+    foreach (QString expr, m_watchVariables.exprList()) {
+	varName.sprintf(ExprFmt, watchNum++);
+	wg.writeEntry(varName, expr);
     }
 
     // give others a chance
@@ -748,48 +758,46 @@ void KDebugger::saveProgramSettings()
 void KDebugger::overrideProgramArguments(const QString& args)
 {
     ASSERT(m_programConfig != 0);
-    m_programConfig->setGroup(GeneralGroup);
-    m_programConfig->writeEntry(ProgramArgs, args);
+    KConfigGroup g = m_programConfig->group(GeneralGroup);
+    g.writeEntry(ProgramArgs, args);
 }
 
 void KDebugger::restoreProgramSettings()
 {
     ASSERT(m_programConfig != 0);
-    m_programConfig->setGroup(GeneralGroup);
+    KConfigGroup gg = m_programConfig->group(GeneralGroup);
     /*
      * We ignore file version for now we will use it in the future to
      * distinguish different versions of this configuration file.
      */
     // m_debuggerCmd has been read in already
     // m_ttyLevel has been read in already
-    QString pgmArgs = m_programConfig->readEntry(ProgramArgs);
-    QString pgmWd = m_programConfig->readEntry(WorkingDirectory);
-    QStringList boolOptions = m_programConfig->readListEntry(OptionsSelected);
-    m_boolOptions = QStringList();
+    QString pgmArgs = gg.readEntry(ProgramArgs);
+    QString pgmWd = gg.readEntry(WorkingDirectory);
+    QSet<QString> boolOptions = QSet<QString>::fromList(gg.readEntry(OptionsSelected, QStringList()));
+    m_boolOptions.clear();
 
     // read environment variables
-    m_programConfig->setGroup(EnvironmentGroup);
+    KConfigGroup eg = m_programConfig->group(EnvironmentGroup);
     m_envVars.clear();
-    QDict<EnvVar> pgmVars;
-    EnvVar* var;
+    std::map<QString,EnvVar> pgmVars;
     QString varName;
     QString varValue;
     for (int i = 0;; ++i) {
 	varName.sprintf(Variable, i);
 	varValue.sprintf(Value, i);
-	if (!m_programConfig->hasKey(varName)) {
+	if (!eg.hasKey(varName)) {
 	    /* entry not present, assume that we've hit them all */
 	    break;
 	}
-	QString name = m_programConfig->readEntry(varName);
+	QString name = eg.readEntry(varName, QString());
 	if (name.isEmpty()) {
 	    // skip empty names
 	    continue;
 	}
-	var = new EnvVar;
-	var->value = m_programConfig->readEntry(varValue);
+	EnvVar* var = &pgmVars[name];
+	var->value = eg.readEntry(varValue, QString());
 	var->status = EnvVar::EVnew;
-	pgmVars.insert(name, var);
     }
 
     updateProgEnvironment(pgmArgs, pgmWd, pgmVars, boolOptions);
@@ -797,15 +805,15 @@ void KDebugger::restoreProgramSettings()
     restoreBreakpoints(m_programConfig);
 
     // watch expressions
-    m_programConfig->setGroup(WatchGroup);
+    KConfigGroup wg = m_programConfig->group(WatchGroup);
     m_watchVariables.clear();
     for (int i = 0;; ++i) {
 	varName.sprintf(ExprFmt, i);
-	if (!m_programConfig->hasKey(varName)) {
+	if (!wg.hasKey(varName)) {
 	    /* entry not present, assume that we've hit them all */
 	    break;
 	}
-	QString expr = m_programConfig->readEntry(varName);
+	QString expr = wg.readEntry(varName, QString());
 	if (expr.isEmpty()) {
 	    // skip empty expressions
 	    continue;
@@ -821,9 +829,9 @@ void KDebugger::restoreProgramSettings()
  * Reads the debugger command line from the program settings. The config
  * group must have been set by the caller.
  */
-QString KDebugger::readDebuggerCmd()
+QString KDebugger::readDebuggerCmd(const KConfigGroup& g)
 {
-    QString debuggerCmd = m_programConfig->readEntry(DebuggerCmdStr);
+    QString debuggerCmd = g.readEntry(DebuggerCmdStr);
 
     // always let the user confirm the debugger cmd if we are root
     if (::geteuid() == 0)
@@ -856,7 +864,7 @@ const char Temporary[] = "Temporary";
 const char Enabled[] = "Enabled";
 const char Condition[] = "Condition";
 
-void KDebugger::saveBreakpoints(ProgramConfig* config)
+void KDebugger::saveBreakpoints(KConfig* config)
 {
     QString groupName;
     int i = 0;
@@ -869,7 +877,7 @@ void KDebugger::saveBreakpoints(ProgramConfig* config)
 	/* remove remmants */
 	config->deleteGroup(groupName);
 
-	config->setGroup(groupName);
+	KConfigGroup g = config->group(groupName);
 	if (!bp->text.isEmpty()) {
 	    /*
 	     * The breakpoint was set using the text box in the breakpoint
@@ -877,29 +885,28 @@ void KDebugger::saveBreakpoints(ProgramConfig* config)
 	     * but instead honor what the user typed (a function name, for
 	     * example, which could move between sessions).
 	     */
-	    config->writeEntry(Text, bp->text);
+	    g.writeEntry(Text, bp->text);
 	} else if (!bp->fileName.isEmpty()) {
-	    config->writeEntry(File, bp->fileName);
-	    config->writeEntry(Line, bp->lineNo);
+	    g.writeEntry(File, bp->fileName);
+	    g.writeEntry(Line, bp->lineNo);
 	    /*
 	     * Addresses are hardly correct across sessions, so we don't
 	     * save it.
 	     */
 	} else {
-	    config->writeEntry(Address, bp->address.asString());
+	    g.writeEntry(Address, bp->address.asString());
 	}
-	config->writeEntry(Temporary, bp->temporary);
-	config->writeEntry(Enabled, bp->enabled);
+	g.writeEntry(Temporary, bp->temporary);
+	g.writeEntry(Enabled, bp->enabled);
 	if (!bp->condition.isEmpty())
-	    config->writeEntry(Condition, bp->condition);
+	    g.writeEntry(Condition, bp->condition);
 	// we do not save the ignore count
     }
     // delete remaining groups
     // we recognize that a group is present if there is an Enabled entry
     for (;; i++) {
 	groupName.sprintf(BPGroup, i);
-	config->setGroup(groupName);
-	if (!config->hasKey(Enabled)) {
+	if (!config->group(groupName).hasKey(Enabled)) {
 	    /* group not present, assume that we've hit them all */
 	    break;
 	}
@@ -907,7 +914,7 @@ void KDebugger::saveBreakpoints(ProgramConfig* config)
     }
 }
 
-void KDebugger::restoreBreakpoints(ProgramConfig* config)
+void KDebugger::restoreBreakpoints(KConfig* config)
 {
     QString groupName;
     /*
@@ -916,16 +923,16 @@ void KDebugger::restoreBreakpoints(ProgramConfig* config)
      */
     for (int i = 0;; i++) {
 	groupName.sprintf(BPGroup, i);
-	config->setGroup(groupName);
-	if (!config->hasKey(Enabled)) {
+	KConfigGroup g = config->group(groupName);
+	if (!g.hasKey(Enabled)) {
 	    /* group not present, assume that we've hit them all */
 	    break;
 	}
 	Breakpoint* bp = new Breakpoint;
-	bp->fileName = config->readEntry(File);
-	bp->lineNo = config->readNumEntry(Line, -1);
-	bp->text = config->readEntry(Text);
-	bp->address = config->readEntry(Address);
+	bp->fileName = g.readEntry(File);
+	bp->lineNo = g.readEntry(Line, -1);
+	bp->text = g.readEntry(Text);
+	bp->address = g.readEntry(Address);
 	// check consistency
 	if ((bp->fileName.isEmpty() || bp->lineNo < 0) &&
 	    bp->text.isEmpty() &&
@@ -934,9 +941,9 @@ void KDebugger::restoreBreakpoints(ProgramConfig* config)
 	    delete bp;
 	    continue;
 	}
-	bp->enabled = config->readBoolEntry(Enabled, true);
-	bp->temporary = config->readBoolEntry(Temporary, false);
-	bp->condition = config->readEntry(Condition);
+	bp->enabled = g.readEntry(Enabled, true);
+	bp->temporary = g.readEntry(Temporary, false);
+	bp->condition = g.readEntry(Condition);
 
 	/*
 	 * Add the breakpoint.
@@ -1230,47 +1237,41 @@ void KDebugger::updateAllExprs()
     }
 
     // update watch expressions
-    VarTree* item = m_watchVariables.firstChild();
-    for (; item != 0; item = item->nextSibling()) {
-	m_watchEvalExpr.push_back(item->getText());
+    foreach (QString expr, m_watchVariables.exprList()) {
+	m_watchEvalExpr.push_back(expr);
     }
 }
 
 void KDebugger::updateProgEnvironment(const QString& args, const QString& wd,
-				      const QDict<EnvVar>& newVars,
-				      const QStringList& newOptions)
+				      const std::map<QString,EnvVar>& newVars,
+				      const QSet<QString>& newOptions)
 {
     m_programArgs = args;
     m_d->executeCmd(DCsetargs, m_programArgs);
     TRACE("new pgm args: " + m_programArgs + "\n");
 
-    m_programWD = wd.stripWhiteSpace();
+    m_programWD = wd.trimmed();
     if (!m_programWD.isEmpty()) {
 	m_d->executeCmd(DCcd, m_programWD);
 	TRACE("new wd: " + m_programWD + "\n");
     }
 
     // update environment variables
-    QDictIterator<EnvVar> it = newVars;
-    EnvVar* val;
-    for (; (val = it) != 0; ++it) {
-	QString var = it.currentKey();
+    for (std::map<QString,EnvVar>::const_iterator i = newVars.begin(); i != newVars.end(); ++i)
+    {
+	QString var = i->first;
+	const EnvVar* val = &i->second;
 	switch (val->status) {
 	case EnvVar::EVnew:
-	    m_envVars.insert(var, val);
-	    // fall thru
 	case EnvVar::EVdirty:
-	    // the value must be in our list
-	    ASSERT(m_envVars[var] == val);
+	    m_envVars[var] = val->value;
 	    // update value
 	    m_d->executeCmd(DCsetenv, var, val->value);
 	    break;
 	case EnvVar::EVdeleted:
-	    // must be in our list
-	    ASSERT(m_envVars[var] == val);
 	    // delete value
 	    m_d->executeCmd(DCunsetenv, var);
-	    m_envVars.remove(var);
+	    m_envVars.erase(var);
 	    break;
 	default:
 	    ASSERT(false);
@@ -1281,27 +1282,17 @@ void KDebugger::updateProgEnvironment(const QString& args, const QString& wd,
     }
 
     // update options
-    QStringList::ConstIterator oi;
-    for (oi = newOptions.begin(); oi != newOptions.end(); ++oi)
+    foreach (QString opt, newOptions - m_boolOptions)
     {
-	if (m_boolOptions.findIndex(*oi) < 0) {
-	    // the options is currently not set, so set it
-	    m_d->executeCmd(DCsetoption, *oi, 1);
-	} else {
-	    // option is set, no action required, but move it to the end
-	    m_boolOptions.remove(*oi);
-	}
-	m_boolOptions.append(*oi);
+	// option is not set, set it
+	m_d->executeCmd(DCsetoption, opt, 1);
     }
-    /*
-     * Now all options that should be set are at the end of m_boolOptions.
-     * If some options need to be unset, they are at the front of the list.
-     * Here we unset and remove them.
-     */
-    while (m_boolOptions.count() > newOptions.count()) {
-	m_d->executeCmd(DCsetoption, m_boolOptions.first(), 0);
-	m_boolOptions.remove(m_boolOptions.begin());
+    foreach (QString opt, m_boolOptions - newOptions)
+    {
+	// option is set, unset it
+	m_d->executeCmd(DCsetoption, opt, 0);
     }
+    m_boolOptions = newOptions;
 }
 
 void KDebugger::handleLocals(const char* output)
@@ -1450,9 +1441,6 @@ bool KDebugger::handlePrintDeref(CmdQueueItem* cmd, const char* output)
 // parse the output of bt
 void KDebugger::handleBacktrace(const char* output)
 {
-    // reduce flicker
-    m_btWindow.setAutoUpdate(false);
-
     m_btWindow.clear();
 
     std::list<StackFrame> stack;
@@ -1470,14 +1458,13 @@ void KDebugger::handleBacktrace(const char* output)
 		func = frm->var->m_name;
 	    else
 		func = frm->fileName + ":" + QString().setNum(frm->lineNo+1);
-	    m_btWindow.insertItem(func);
+        
+ 	    m_btWindow.addItem(func);
 	    TRACE("frame " + func + " (" + frm->fileName + ":" +
 		  QString().setNum(frm->lineNo+1) + ")");
 	}
     }
 
-    m_btWindow.setAutoUpdate(true);
-    m_btWindow.repaint();
 }
 
 void KDebugger::gotoFrame(int frame)
@@ -1610,7 +1597,7 @@ void KDebugger::handleFindType(CmdQueueItem* cmd, const char* output)
     {
 	ASSERT(cmd != 0 && cmd->m_expr != 0);
 
-	TypeInfo* info = m_typeTable->lookup(type);
+	const TypeInfo* info = m_typeTable->lookup(type);
 
 	if (info == 0) {
 	    /*
@@ -1803,20 +1790,20 @@ CmdQueueItem* KDebugger::loadCoreFile()
     return m_d->queueCmd(DCcorefile, m_corefile, DebuggerDriver::QMoverride);
 }
 
-void KDebugger::slotExpanding(QListViewItem* item)
+void KDebugger::slotExpanding(QTreeWidgetItem* item)
 {
     VarTree* exprItem = static_cast<VarTree*>(item);
     if (exprItem->m_varKind != VarTree::VKpointer) {
 	return;
     }
-    ExprWnd* wnd = static_cast<ExprWnd*>(item->listView());
+    ExprWnd* wnd = static_cast<ExprWnd*>(item->treeWidget());
     dereferencePointer(wnd, exprItem, true);
 }
 
 // add the expression in the edit field to the watch expressions
 void KDebugger::addWatch(const QString& t)
 {
-    QString expr = t.stripWhiteSpace();
+    QString expr = t.trimmed();
     // don't add a watched expression again
     if (expr.isEmpty() || m_watchVariables.topLevelExprByName(expr) != 0)
 	return;
@@ -1840,12 +1827,13 @@ void KDebugger::slotDeleteWatch()
     if (m_d == 0 || !m_d->isIdle())
 	return;
 
-    VarTree* item = m_watchVariables.currentItem();
+    VarTree* item = m_watchVariables.selectedItem();
     if (item == 0 || !item->isToplevelExpr())
 	return;
 
     // remove the variable from the list to evaluate
-    QStringList::iterator i = m_watchEvalExpr.find(item->getText());
+    std::list<QString>::iterator i =
+	    std::find(m_watchEvalExpr.begin(), m_watchEvalExpr.end(), item->getText());
     if (i != m_watchEvalExpr.end()) {
 	m_watchEvalExpr.erase(i);
     }
@@ -1993,18 +1981,12 @@ KDebugger::BrkptIterator KDebugger::breakpointByFilePos(QString file, int lineNo
 	}
     }
     // not found, so try basename
-    // strip off directory part of file name
-    int offset = file.findRev("/");
-    file.remove(0, offset+1);
+    file = QFileInfo(file).fileName();
 
     for (BrkptIterator bp = m_brkpts.begin(); bp != m_brkpts.end(); ++bp)
     {
 	// get base name of breakpoint's file
-	QString basename = bp->fileName;
-	int offset = basename.findRev("/");
-	if (offset >= 0) {
-	    basename.remove(0, offset+1);
-	}
+	QString basename = QFileInfo(bp->fileName).fileName();
 
 	if (bp->lineNo == lineNo &&
 	    basename == file &&
@@ -2173,10 +2155,10 @@ void KDebugger::handleSetPC(const char* /*output*/)
 
 void KDebugger::slotValueEdited(VarTree* expr, const QString& text)
 {
-    if (text.simplifyWhiteSpace().isEmpty())
+    if (text.simplified().isEmpty())
 	return;			       /* no text entered: ignore request */
 
-    ExprWnd* wnd = static_cast<ExprWnd*>(expr->listView());
+    ExprWnd* wnd = static_cast<ExprWnd*>(expr->treeWidget());
     TRACE(QString().sprintf("Changing %s to ",
 			    wnd->name()) + text);
 
