@@ -6,17 +6,16 @@
 
 #include "gdbdriver.h"
 #include "exprwnd.h"
-#include <qregexp.h>
-#include <qstringlist.h>
+#include <QFileInfo>
+#include <QRegExp>
+#include <QStringList>
 #include <klocale.h>			/* i18n */
 #include <ctype.h>
+#include <signal.h>
 #include <stdlib.h>			/* strtol, atoi */
 #include <string.h>			/* strcpy */
 
 #include "assert.h"
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "mydebug.h"
 
 static void skipString(const char*& p);
@@ -30,8 +29,6 @@ static bool parseValueSeq(const char*& s, ExprValue* variable);
 
 #define PROMPT "(kdbg)"
 #define PROMPT_LEN 6
-#define PROMPT_LAST_CHAR ')'		/* needed when searching for prompt string */
-
 
 // TODO: make this cmd info stuff non-static to allow multiple
 // simultaneous gdbs to run!
@@ -71,7 +68,11 @@ static GdbCmdInfo cmds[] = {
     { DCtty, "tty %s\n", GdbCmdInfo::argString },
     { DCexecutable, "file \"%s\"\n", GdbCmdInfo::argString },
     { DCtargetremote, "target remote %s\n", GdbCmdInfo::argString },
-    { DCcorefile, "core-file %s\n", GdbCmdInfo::argString },
+#ifdef __FreeBSD__
+    { DCcorefile, "target FreeBSD-core %s\n", GdbCmdInfo::argString },
+#else
+    { DCcorefile, "target core %s\n", GdbCmdInfo::argString },
+#endif
     { DCattach, "attach %s\n", GdbCmdInfo::argString },
     { DCinfolinemain, "kdbg_infolinemain\n", GdbCmdInfo::argNone },
     { DCinfolocals, "kdbg__alllocals\n", GdbCmdInfo::argNone },
@@ -124,13 +125,8 @@ static GdbCmdInfo cmds[] = {
 #define MAX_FMTLEN 200
 
 GdbDriver::GdbDriver() :
-	DebuggerDriver(),
-	m_gdbMajor(4), m_gdbMinor(16)
+	DebuggerDriver()
 {
-    strcpy(m_prompt, PROMPT);
-    m_promptMinLen = PROMPT_LEN;
-    m_promptLastChar = PROMPT_LAST_CHAR;
-
 #ifndef NDEBUG
     // check command info array
     const char* perc;
@@ -281,37 +277,6 @@ void GdbDriver::commandFinished(CmdQueueItem* cmd)
 
     switch (cmd->m_cmd) {
     case DCinitialize:
-	// get version number from preamble
-	{
-	    int len;
-	    QRegExp GDBVersion("\\nGDB [0-9]+\\.[0-9]+");
-	    int offset = GDBVersion.match(m_output, 0, &len);
-	    if (offset >= 0) {
-		char* start = m_output + offset + 5;	// skip "\nGDB "
-		char* end;
-		m_gdbMajor = strtol(start, &end, 10);
-		m_gdbMinor = strtol(end + 1, 0, 10);	// skip "."
-		if (start == end) {
-		    // nothing was parsed
-		    m_gdbMajor = 4;
-		    m_gdbMinor = 16;
-		}
-	    } else {
-		// assume some default version (what would make sense?)
-		m_gdbMajor = 4;
-		m_gdbMinor = 16;
-	    }
-	    // use a feasible core-file command
-	    if (m_gdbMajor > 4 || (m_gdbMajor == 4 && m_gdbMinor >= 16)) {
-#ifdef __FreeBSD__
-		cmds[DCcorefile].fmt = "target FreeBSD-core %s\n";
-#else
-		cmds[DCcorefile].fmt = "target core %s\n";
-#endif
-	    } else {
-		cmds[DCcorefile].fmt = "core-file %s\n";
-	    }
-	}
 	{
 	    /*
 	     * Check for GDB 7.1 or later; the syntax for the disassemble
@@ -326,7 +291,7 @@ void GdbDriver::commandFinished(CmdQueueItem* cmd)
 		"(\\d+)\\.(\\d+)"	// major, minor
 		"[^ ]*\\n"		// no space until end of line
 		);
-	    int pos = re.search(m_output);
+	    int pos = re.indexIn(m_output);
 	    const char* disass = "disassemble %s %s\n";
 	    if (pos >= 0) {
 		int major = re.cap(1).toInt();
@@ -343,11 +308,12 @@ void GdbDriver::commandFinished(CmdQueueItem* cmd)
     }
 
     /* ok, the command is ready */
-    emit commandReceived(cmd, m_output);
+    emit commandReceived(cmd, m_output.constData());
 
     switch (cmd->m_cmd) {
     case DCcorefile:
     case DCinfolinemain:
+    case DCinfoline:
     case DCframe:
     case DCattach:
     case DCrun:
@@ -358,19 +324,41 @@ void GdbDriver::commandFinished(CmdQueueItem* cmd)
     case DCnexti:
     case DCfinish:
     case DCuntil:
-	parseMarker();
+	parseMarker(cmd);
     default:;
     }
 }
 
+int GdbDriver::findPrompt(const QByteArray& output) const
+{
+    /*
+     * If there's a prompt string in the collected output, it must be at
+     * the very end.
+     * 
+     * Note: It could nevertheless happen that a character sequence that is
+     * equal to the prompt string appears at the end of the output,
+     * although it is very, very unlikely (namely as part of a string that
+     * lingered in gdb's output buffer due to some timing/heavy load
+     * conditions for a very long time such that that buffer overflowed
+     * exactly at the end of the prompt string look-a-like).
+     */
+    int len = output.length();
+    if (len >= PROMPT_LEN &&
+	strncmp(output.data()+len-PROMPT_LEN, PROMPT, PROMPT_LEN) == 0)
+    {
+	return len-PROMPT_LEN;
+    }
+    return -1;
+}
+
 /*
  * The --fullname option makes gdb send a special normalized sequence print
  * each time the program stops and at some other points. The sequence has
  * the form "\032\032filename:lineno:charoffset:(beg|middle):address".
  */
-void GdbDriver::parseMarker()
+void GdbDriver::parseMarker(CmdQueueItem* cmd)
 {
-    char* startMarker = strstr(m_output, "\032\032");
+    char* startMarker = strstr(m_output.data(), "\032\032");
     if (startMarker == 0)
 	return;
 
@@ -384,16 +372,19 @@ void GdbDriver::parseMarker()
     *endMarker = '\0';
 
     // extract filename and line number
-    static QRegExp MarkerRE(":[0-9]+:[0-9]+:[begmidl]+:0x");
+    static QRegExp MarkerRE(":(\\d+):\\d+:[begmidl]+:0x");
 
-    int len;
-    int lineNoStart = MarkerRE.match(startMarker, 0, &len);
+    int lineNoStart = MarkerRE.indexIn(startMarker);
     if (lineNoStart >= 0) {
-	int lineNo = atoi(startMarker + lineNoStart+1);
+	int lineNo = MarkerRE.cap(1).toInt();
 
-	// get address
-	const char* addrStart = startMarker + lineNoStart + len - 2;
-	DbgAddr address = QString(addrStart).stripWhiteSpace();
+	// get address unless there is one in cmd
+	DbgAddr address = cmd->m_addr;
+	if (address.isEmpty()) {
+	    const char* addrStart = startMarker + lineNoStart +
+				    MarkerRE.matchedLength() - 2;
+	    address = QString(addrStart).trimmed();
+	}
 
 	// now show the window
 	startMarker[lineNoStart] = '\0';   /* split off file name */
@@ -438,7 +429,7 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg)
     }
 
     QString cmdString;
-    cmdString.sprintf(cmds[cmd].fmt, strArg.latin1());
+    cmdString.sprintf(cmds[cmd].fmt, strArg.toLatin1().constData());
     return cmdString;
 }
 
@@ -542,15 +533,13 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg, int intArg)
 	if (cmd == DCinfoline)
 	{
 	    // must split off file name part
-	    int slash = strArg.findRev('/');
-	    if (slash >= 0)
-		strArg = strArg.right(strArg.length()-slash-1);
+	    strArg = QFileInfo(strArg).fileName();
 	}
-	cmdString.sprintf(cmds[cmd].fmt, strArg.latin1(), intArg);
+	cmdString.sprintf(cmds[cmd].fmt, strArg.toLatin1().constData(), intArg);
     }
     else
     {
-	cmdString.sprintf(cmds[cmd].fmt, intArg, strArg.latin1());
+	cmdString.sprintf(cmds[cmd].fmt, intArg, strArg.toLatin1().constData());
     }
     return cmdString;
 }
@@ -564,7 +553,9 @@ QString GdbDriver::makeCmdString(DbgCommand cmd, QString strArg1, QString strArg
     normalizeStringArg(strArg2);
 
     QString cmdString;
-    cmdString.sprintf(cmds[cmd].fmt, strArg1.latin1(), strArg2.latin1());
+    cmdString.sprintf(cmds[cmd].fmt,
+		      strArg1.toLatin1().constData(),
+		      strArg2.toLatin1().constData());
     return cmdString;
 }
 
@@ -652,20 +643,20 @@ CmdQueueItem* GdbDriver::queueCmd(DbgCommand cmd, QString strArg1, QString strAr
 
 void GdbDriver::terminate()
 {
-    kill(SIGTERM);
+    ::kill(pid(), SIGTERM);
     m_state = DSidle;
 }
 
 void GdbDriver::detachAndTerminate()
 {
-    kill(SIGINT);
+    ::kill(pid(), SIGINT);
     flushCommands();
     executeCmdString(DCinitialize, "detach\nquit\n", true);
 }
 
 void GdbDriver::interruptInferior()
 {
-    kill(SIGINT);
+    ::kill(pid(), SIGINT);
     // remove accidentally queued commands
     flushHiPriQueue();
 }
@@ -834,7 +825,7 @@ ExprValue* GdbDriver::parseQCharArray(const char* output, bool wantErrorValue, b
 
 	    // escape a few frequently used characters
 	    char escapeCode = '\0';
-	    switch (ch.latin1()) {
+	    switch (ch.toLatin1()) {
 	    case '\n': escapeCode = 'n'; break;
 	    case '\r': escapeCode = 'r'; break;
 	    case '\t': escapeCode = 't'; break;
@@ -930,7 +921,7 @@ static ExprValue* parseVar(const char*& s)
 	while (isspace(*p))
 	    p++;
 	if (*p != '=') {
-	    TRACE(QString().sprintf("parse error: = not found after %s", (const char*)name));
+	    TRACE("parse error: = not found after " + name);
 	    return 0;
 	}
 	// skip the '=' and more whitespace
@@ -1076,7 +1067,8 @@ moreStrings:
 	}
     }
     // is the string continued?
-    if (*p == ',') {
+    if (*p == ',')
+    {
 	// look ahead for another quote
 	const char* q = p+1;
 	while (isspace(*q))
@@ -1086,6 +1078,17 @@ moreStrings:
 	    p = q;
 	    goto moreStrings;
 	}
+
+	// some strings can end in <incomplete sequence ...>
+	if (strncmp(q, "<incomplete sequence", 20) == 0)
+	{
+	    p = q+20;
+	    while (*p != '\0' && *p != '>')
+		p++;
+	    if (*p != '\0') {
+		p++;		/* skip the '>' */
+	    }
+	}
     }
     /* very long strings are followed by `...' */
     if (*p == '.' && p[1] == '.' && p[2] == '.') {
@@ -1211,6 +1214,15 @@ repeat:
 	    while (isspace(*s))
 		s++;
 	}
+    }
+    // Sometimes we find a warning; it ends at the next LF
+    else if (strncmp(s, "warning: ", 9) == 0) {
+	const char* end = strchr(s, '\n');
+	s = end ? end : s+strlen(s);
+	// skip space at start of next line
+	while (isspace(*s))
+	    s++;
+	goto repeat;
     } else {
 	// examples of leaf values (cannot be the empty string):
 	//  123
@@ -1666,23 +1678,23 @@ static void parseFrameInfo(const char*& s, QString& func,
 
     /*
      * Replace \n (and whitespace around it) in func by a blank. We cannot
-     * use QString::simplifyWhiteSpace() for this because this would also
+     * use QString::simplified() for this because this would also
      * simplify space that belongs to a string arguments that gdb sometimes
      * prints in the argument lists of the function.
      */
-    ASSERT(!isspace(func[0].latin1()));		/* there must be non-white before first \n */
+    ASSERT(!isspace(func[0].toLatin1()));	// there must be non-white before first \n
     int nl = 0;
-    while ((nl = func.find('\n', nl)) >= 0) {
+    while ((nl = func.indexOf('\n', nl)) >= 0) {
 	// search back to the beginning of the whitespace
 	int startWhite = nl;
 	do {
 	    --startWhite;
-	} while (isspace(func[startWhite].latin1()));
+	} while (isspace(func[startWhite].toLatin1()));
 	startWhite++;
 	// search forward to the end of the whitespace
 	do {
 	    nl++;
-	} while (isspace(func[nl].latin1()));
+	} while (isspace(func[nl].toLatin1()));
 	// replace
 	func.replace(startWhite, nl-startWhite, " ");
 	/* continue searching for more \n's at this place: */
@@ -1819,7 +1831,7 @@ bool GdbDriver::parseBreakList(const char* output, std::list<Breakpoint>& brks)
 	    bp.location = p;
 	    p += bp.location.length();
 	} else {
-	    bp.location = QString::fromLatin1(p, end-p).stripWhiteSpace();
+	    bp.location = QString::fromLatin1(p, end-p).trimmed();
 	    p = end+1;			/* skip over \n */
 	}
 
@@ -1842,7 +1854,7 @@ bool GdbDriver::parseBreakList(const char* output, std::list<Breakpoint>& brks)
 	    } else if (strncmp(p, "stop only if ", 13) == 0) {
 		// extract condition
 		p += 13;
-		bp.condition = QString::fromLatin1(p, end-p).stripWhiteSpace();
+		bp.condition = QString::fromLatin1(p, end-p).trimmed();
 		TRACE("condition: "+bp.condition);
 	    } else if (strncmp(p, "ignore next ", 12) == 0) {
 		// extract ignore count
@@ -1851,7 +1863,7 @@ bool GdbDriver::parseBreakList(const char* output, std::list<Breakpoint>& brks)
 		TRACE(QString("ignore count %1").arg(bp.ignoreCount));
 	    } else {
 		// indeed a continuation
-		bp.location += " " + QString::fromLatin1(p, end-p).stripWhiteSpace();
+		bp.location += " " + QString::fromLatin1(p, end-p).trimmed();
 	    }
 	    p = end;
 	    if (*p != '\0')
@@ -1944,6 +1956,9 @@ bool GdbDriver::parseBreakpoint(const char* output, int& id,
     if (strncmp(o, "Breakpoint ", 11) == 0) {
 	output += 11;			/* skip "Breakpoint " */
 	return ::parseNewBreakpoint(output, id, file, lineNo, address);
+    } else if (strncmp(o, "Temporary breakpoint ", 21) == 0) {
+	output += 21;
+	return ::parseNewBreakpoint(output, id, file, lineNo, address);
     } else if (strncmp(o, "Hardware watchpoint ", 20) == 0) {
 	output += 20;
 	return ::parseNewWatchpoint(output, id, address);
@@ -2005,7 +2020,7 @@ static bool parseNewWatchpoint(const char* o, int& id,
     p += 2;
 
     // all the rest on the line is the expression
-    expr = QString::fromLatin1(p, strlen(p)).stripWhiteSpace();
+    expr = QString::fromLatin1(p, strlen(p)).trimmed();
     return true;
 }
 
@@ -2064,7 +2079,7 @@ ExprValue* GdbDriver::parsePrintExpr(const char* output, bool wantErrorValue)
 bool GdbDriver::parseChangeWD(const char* output, QString& message)
 {
     bool isGood = false;
-    message = QString(output).simplifyWhiteSpace();
+    message = QString(output).simplified();
     if (message.isEmpty()) {
 	message = i18n("New working directory: ") + m_programWD;
 	isGood = true;
@@ -2236,8 +2251,6 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 	return regs;
     }
 
-    QString value;
-
     // parse register values
     while (*output != '\0')
     {
@@ -2258,6 +2271,8 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 	while (isspace(*output))
 	    output++;
 
+	QString value;
+
 	/*
 	 * If we find a brace now, this is a vector register. We look for
 	 * the closing brace and treat the result as cooked value.
@@ -2266,7 +2281,7 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 	{
 	    start = output;
 	    skipNested(output, '{', '}');
-	    value = QString::fromLatin1(start, output-start).simplifyWhiteSpace();
+	    value = QString::fromLatin1(start, output-start).simplified();
 	    // skip space, but not the end of line
 	    while (isspace(*output) && *output != '\n')
 		output++;
@@ -2283,7 +2298,7 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 
 		start = output;
 		skipNested(output, '{', '}');
-		value = QString::fromLatin1(start, output-start).simplifyWhiteSpace();
+		value = QString::fromLatin1(start, output-start).simplified();
 	    } else {
 		// for gdb 5.3
 		// find first type that does not have an array, this is the RAW value
@@ -2300,7 +2315,7 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 			end=cur;
 			while (*end && (*end!='}') && (*end!=',') && (*end!='\n'))
 			    end++;
-			QString rawValue = QString::fromLatin1(cur, end-cur).simplifyWhiteSpace();
+			QString rawValue = QString::fromLatin1(cur, end-cur).simplified();
 			reg.rawValue = rawValue;
 
 			if (rawValue.left(2)=="0x") {
@@ -2332,12 +2347,24 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 	}
 	else
 	{
+	continuation:
 	    // the rest of the line is the register value
 	    start = output;
 	    output = strchr(output,'\n');
 	    if (output == 0)
 		output = start + strlen(start);
-	    value = QString::fromLatin1(start, output-start).simplifyWhiteSpace();
+	    value += QString::fromLatin1(start, output-start).simplified();
+
+	    /*
+	     * Look ahead: if the subsequent line is indented, it continues
+	     * the current register value.
+	     */
+	    if (output != 0 && isspace(output[1]))
+	    {
+		++output;
+		value += ' ';
+		goto continuation;
+	    }
 
 	    /*
 	     * We split the raw from the cooked values.
@@ -2345,7 +2372,7 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 	     * Here, the cooked value comes first, and the raw value is in
 	     * the second part.
 	     */
-	    int pos = value.find(" (raw ");
+	    int pos = value.indexOf(" (raw ");
 	    if (pos >= 0)
 	    {
 		reg.cookedValue = value.left(pos);
@@ -2360,7 +2387,7 @@ std::list<RegisterInfo> GdbDriver::parseRegisters(const char* output)
 		* whitespace). It is the raw value. The remainder of the line
 		* is the cooked value.
 		*/
-		int pos = value.find(' ');
+		int pos = value.indexOf(' ');
 		if (pos < 0) {
 		    reg.rawValue = value;
 		    reg.cookedValue = QString();
@@ -2434,6 +2461,15 @@ std::list<DisassembledCode> GdbDriver::parseDisassemble(const char* output)
     while (p != end)
     {
 	DisassembledCode c;
+	// skip initial space or PC pointer ("=>", since gdb 7.1)
+	while (p != end) {
+	    if (isspace(*p))
+		++p;
+	    else if (p[0] == '=' && p[1] == '>')
+		p += 2;
+	    else
+		break;
+	}
 	const char* start = p;
 	// address
 	while (p != end && !isspace(*p))
@@ -2470,7 +2506,7 @@ QString GdbDriver::parseMemoryDump(const char* output, std::list<MemoryDump>& me
     if (isErrorExpr(output)) {
 	// error; strip space
 	QString msg = output;
-	return msg.stripWhiteSpace();
+	return msg.trimmed();
     }
 
     const char* p = output;		/* save typing */
@@ -2514,7 +2550,8 @@ QString GdbDriver::parseMemoryDump(const char* output, std::list<MemoryDump>& me
 
 QString GdbDriver::editableValue(VarTree* value)
 {
-    const char* s = value->value().latin1();
+    QByteArray ba = value->value().toLatin1();
+    const char* s = ba.constData();
 
     // if the variable is a pointer value that contains a cast,
     // remove the cast
@@ -2565,7 +2602,7 @@ QString GdbDriver::parseSetVariable(const char* output)
 {
     // if there is any output, it is an error message
     QString msg = output;
-    return msg.stripWhiteSpace();
+    return msg.trimmed();
 }
 
 
