@@ -6,31 +6,31 @@
 
 #include "debugger.h"
 #include "sourcewnd.h"
-#include <qtextstream.h>
-#include <qpainter.h>
-#include <qbrush.h>
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <qkeycode.h>
-#include <qpopupmenu.h>
-#include <kapplication.h>
+#include "dbgdriver.h"
+#include <QTextStream>
+#include <QPainter>
+#include <QFile>
+#include <QFileInfo>
+#include <QMenu>
+#include <QContextMenuEvent>
+#include <QKeyEvent>
+#include <QMouseEvent>
 #include <kiconloader.h>
 #include <kglobalsettings.h>
-#include <kmainwindow.h>
+#include <kxmlguiwindow.h>
+#include <kxmlguifactory.h>
 #include <algorithm>
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "mydebug.h"
 
 
-SourceWindow::SourceWindow(const QString& fileName, QWidget* parent, const char* name) :
-	QTextEdit(parent, name),
+SourceWindow::SourceWindow(const QString& fileName, QWidget* parent) :
+	QPlainTextEdit(parent),
 	m_fileName(fileName),
-	m_curRow(-1),
+	m_highlighter(0),
 	m_widthItems(16),
 	m_widthPlus(12),
-	m_widthLineNo(30)
+	m_widthLineNo(30),
+	m_lineInfoArea(new LineInfoArea(this))
 {
     // load pixmaps
     m_pcinner = UserIcon("pcinner");
@@ -42,51 +42,50 @@ SourceWindow::SourceWindow(const QString& fileName, QWidget* parent, const char*
     m_brkorph = UserIcon("brkorph");
     setFont(KGlobalSettings::fixedFont());
     setReadOnly(true);
-    setMargins(m_widthItems+m_widthPlus+m_widthLineNo, 0, 0 ,0);
-    setAutoFormatting(AutoNone);
-    setTextFormat(PlainText);
-    setWordWrap(NoWrap);
-    connect(verticalScrollBar(), SIGNAL(valueChanged(int)),
-	    this, SLOT(update()));
-    connect(this, SIGNAL(cursorPositionChanged(int,int)), this, SLOT(cursorChanged(int)));
-    viewport()->installEventFilter(this);
+    setViewportMargins(lineInfoAreaWidth(), 0, 0 ,0);
+    setWordWrapMode(QTextOption::NoWrap);
+    connect(this, SIGNAL(updateRequest(const QRect&, int)),
+	    m_lineInfoArea, SLOT(update()));
+    connect(this, SIGNAL(cursorPositionChanged()), this, SLOT(cursorChanged()));
 
     // add a syntax highlighter
-    if (QRegExp("\\.(c(pp|c|\\+\\+)?|CC?|h(\\+\\+|h)?|HH?)$").search(m_fileName) >= 0)
+    if (QRegExp("\\.(c(pp|c|\\+\\+)?|CC?|h(\\+\\+|h)?|HH?)$").indexIn(m_fileName) >= 0)
     {
-	new HighlightCpp(this);
+	m_highlighter = new HighlightCpp(this);
     }
 }
 
 SourceWindow::~SourceWindow()
 {
-    delete syntaxHighlighter();
+    delete m_highlighter;
+}
+
+int SourceWindow::lineInfoAreaWidth() const
+{
+    return 3 + m_widthItems + m_widthPlus + m_widthLineNo;
 }
 
 bool SourceWindow::loadFile()
 {
-    // first we load the code into QTextEdit
     QFile f(m_fileName);
-    if (!f.open(IO_ReadOnly)) {
+    if (!f.open(QIODevice::ReadOnly)) {
 	return false;
     }
 
     QTextStream t(&f);
-    setText(t.read());
+    setPlainText(t.readAll());
     f.close();
 
-    // then we copy it into our own m_sourceCode
-    int n = paragraphs();
+    int n = blockCount();
     m_sourceCode.resize(n);
     m_rowToLine.resize(n);
     for (int i = 0; i < n; i++) {
-	m_sourceCode[i].code = text(i);
 	m_rowToLine[i] = i;
     }
     m_lineItems.resize(n, 0);
 
     // set a font for line numbers
-    m_lineNoFont = currentFont();
+    m_lineNoFont = currentCharFormat().font();
     m_lineNoFont.setPixelSize(11);
 
     return true;
@@ -95,25 +94,21 @@ bool SourceWindow::loadFile()
 void SourceWindow::reloadFile()
 {
     QFile f(m_fileName);
-    if (!f.open(IO_ReadOnly)) {
+    if (!f.open(QIODevice::ReadOnly)) {
 	// open failed; leave alone
 	return;
     }
 
-    // read text into m_sourceCode
     m_sourceCode.clear();		/* clear old text */
 
     QTextStream t(&f);
-    setText(t.read());
+    setPlainText(t.readAll());
     f.close();
 
-    m_sourceCode.resize(paragraphs());
-    for (size_t i = 0; i < m_sourceCode.size(); i++) {
-	m_sourceCode[i].code = text(i);
-    }
+    m_sourceCode.resize(blockCount());
     // expanded lines are collapsed: move existing line items up
     for (size_t i = 0; i < m_lineItems.size(); i++) {
-	if (m_rowToLine[i] != i) {
+	if (m_rowToLine[i] != int(i)) {
 	    m_lineItems[m_rowToLine[i]] |= m_lineItems[i];
 	    m_lineItems[i] = 0;
 	}
@@ -128,9 +123,8 @@ void SourceWindow::reloadFile()
     // Highlighting was applied above when the text was inserted into widget,
     // but at that time m_rowToLine was not corrected, yet, so that lines
     // that previously were assembly were painted incorrectly.
-    if (syntaxHighlighter())
-	syntaxHighlighter()->rehighlight();
-    update();	// line numbers
+    if (m_highlighter)
+	m_highlighter->rehighlight();
 }
 
 void SourceWindow::scrollTo(int lineNo, const DbgAddr& address)
@@ -144,43 +138,42 @@ void SourceWindow::scrollTo(int lineNo, const DbgAddr& address)
 
 void SourceWindow::scrollToRow(int row)
 {
-    setCursorPosition(row, 0);
+    QTextCursor cursor(document()->findBlockByNumber(row));
+    setTextCursor(cursor);
     ensureCursorVisible();
 }
 
-void SourceWindow::drawFrame(QPainter* p)
+void SourceWindow::resizeEvent(QResizeEvent* e)
 {
-    QTextEdit::drawFrame(p);
+    QPlainTextEdit::resizeEvent(e);
 
-    // and paragraph at the top is...
-    int top = paragraphAt(QPoint(0,contentsY()));
-    int bot = paragraphAt(QPoint(0,contentsY()+visibleHeight()-1));
-    if (bot < 0)
-	bot = paragraphs()-1;
+    QRect cr = contentsRect();
+    cr.setRight(lineInfoAreaWidth());
+    m_lineInfoArea->setGeometry(cr);
+}
 
-    p->save();
-
-    // set a clip rectangle
-    int fw = frameWidth();
-    QRect inside = rect();
-    inside.addCoords(fw,fw,-fw,-fw);
-    QRegion clip = p->clipRegion();
-    clip &= QRegion(inside);
-    p->setClipRegion(clip);
+void SourceWindow::drawLineInfoArea(QPainter* p, QPaintEvent* event)
+{
+    QTextBlock block = firstVisibleBlock();
 
     p->setFont(m_lineNoFont);
-    p->setPen(colorGroup().text());
-    p->eraseRect(inside);
 
-    for (int row = top; row <= bot; row++)
+    for (; block.isValid(); block = block.next())
     {
-	uchar item = m_lineItems[row];
-	p->save();
+	if (!block.isVisible())
+	    continue;
 
-	QRect r = paragraphRect(row);
-	QPoint pt = contentsToViewport(r.topLeft());
+	QRect r = blockBoundingGeometry(block).translated(contentOffset()).toRect();
+	if (r.top() > event->rect().bottom() || r.bottom() < event->rect().top())
+	    continue;
+
+	int row = block.blockNumber();
+	uchar item = m_lineItems[row];
+
+	QPoint pt = r.topLeft();
 	int h = r.height();
-	p->translate(fw, pt.y()+viewport()->y());
+	p->save();
+	p->translate(0, r.top());
 
 	if (item & liBP) {
 	    // enabled breakpoint
@@ -236,12 +229,11 @@ void SourceWindow::drawFrame(QPainter* p)
 	}
 	p->translate(m_widthPlus, 0);
 	if (!isRowDisassCode(row)) {
-	    p->drawText(0, 0, m_widthLineNo, h, AlignRight|AlignVCenter,
+	    p->drawText(0, 0, m_widthLineNo, h, Qt::AlignRight|Qt::AlignVCenter,
 			QString().setNum(rowToLine(row)+1));
 	}
 	p->restore();
     }
-    p->restore();
 }
 
 void SourceWindow::updateLineItems(const KDebugger* dbg)
@@ -266,7 +258,6 @@ void SourceWindow::updateLineItems(const KDebugger* dbg)
 	    if (bp == dbg->breakpointsEnd()) {
 		/* doesn't exist anymore, remove it */
 		m_lineItems[i] &= ~liBPany;
-		update();
 	    }
 	}
     }
@@ -275,7 +266,7 @@ void SourceWindow::updateLineItems(const KDebugger* dbg)
     for (KDebugger::BrkptROIterator bp = dbg->breakpointsBegin(); bp != dbg->breakpointsEnd(); ++bp)
     {
 	if (fileNameMatches(bp->fileName)) {
-	    TRACE(QString().sprintf("updating %s:%d", bp->fileName.data(), bp->lineNo));
+	    TRACE(QString("updating %2:%1").arg(bp->lineNo).arg(bp->fileName));
 	    int i = bp->lineNo;
 	    if (i < 0 || i >= int(m_sourceCode.size()))
 		continue;
@@ -292,10 +283,10 @@ void SourceWindow::updateLineItems(const KDebugger* dbg)
 	    if ((m_lineItems[row] & liBPany) != flags) {
 		m_lineItems[row] &= ~liBPany;
 		m_lineItems[row] |= flags;
-		update();
 	    }
 	}
     }
+    m_lineInfoArea->update();
 }
 
 void SourceWindow::setPC(bool set, int lineNo, const DbgAddr& address, int frameNo)
@@ -311,13 +302,13 @@ void SourceWindow::setPC(bool set, int lineNo, const DbgAddr& address, int frame
 	// set only if not already set
 	if ((m_lineItems[row] & flag) == 0) {
 	    m_lineItems[row] |= flag;
-	    update();
+	    m_lineInfoArea->update();
 	}
     } else {
 	// clear only if not set
 	if ((m_lineItems[row] & flag) != 0) {
 	    m_lineItems[row] &= ~flag;
-	    update();
+	    m_lineInfoArea->update();
 	}
     }
 }
@@ -325,29 +316,36 @@ void SourceWindow::setPC(bool set, int lineNo, const DbgAddr& address, int frame
 void SourceWindow::find(const QString& text, bool caseSensitive, FindDirection dir)
 {
     ASSERT(dir == 1 || dir == -1);
-    if (QTextEdit::find(text, caseSensitive, false, dir > 0))
+    QTextDocument::FindFlags flags = 0;
+    if (caseSensitive)
+	flags |= QTextDocument::FindCaseSensitively;
+    if (dir < 0)
+	flags |= QTextDocument::FindBackward;
+    if (QPlainTextEdit::find(text, flags))
 	return;
     // not found; wrap around
-    int para = dir > 0 ? 0 : paragraphs(), index = 0;
-    QTextEdit::find(text, caseSensitive, false, dir > 0, &para, &index);
+    QTextCursor cursor(document());
+    if (dir < 0)
+	cursor.movePosition(QTextCursor::End);
+    cursor = document()->find(text, cursor, flags);
+    if (!cursor.isNull())
+	setTextCursor(cursor);
 }
 
-void SourceWindow::mousePressEvent(QMouseEvent* ev)
+void SourceWindow::infoMousePress(QMouseEvent* ev)
 {
     // we handle left and middle button
-    if (ev->button() != LeftButton && ev->button() != MidButton)
+    if (ev->button() != Qt::LeftButton && ev->button() != Qt::MidButton)
     {
-	QTextEdit::mousePressEvent(ev);
 	return;
     }
 
     // get row
-    QPoint p = viewportToContents(QPoint(0, ev->y() - viewport()->y()));
-    int row = paragraphAt(p);
+    int row = cursorForPosition(QPoint(0, ev->y())).blockNumber();
     if (row < 0)
 	return;
 
-    if (ev->x() > m_widthItems+frameWidth())
+    if (ev->x() > m_widthItems)
     {
 	if (isRowExpanded(row)) {
 	    actionCollapseRow(row);
@@ -369,12 +367,12 @@ void SourceWindow::mousePressEvent(QMouseEvent* ev)
     }
 
     switch (ev->button()) {
-    case LeftButton:
+    case Qt::LeftButton:
 	TRACE(QString().sprintf("left-clicked line %d", line));
 	emit clickedLeft(m_fileName, line, address,
-			 (ev->state() & ShiftButton) != 0);
+ 			 (ev->modifiers() & Qt::ShiftModifier) != 0);
 	break;
-    case MidButton:
+    case Qt::MidButton:
 	TRACE(QString().sprintf("mid-clicked row %d", line));
 	emit clickedMid(m_fileName, line, address);
 	break;
@@ -384,82 +382,77 @@ void SourceWindow::mousePressEvent(QMouseEvent* ev)
 
 void SourceWindow::keyPressEvent(QKeyEvent* ev)
 {
-    int top1, top2;
-    QPoint top;
+    int top1 = 0, top2;
     switch (ev->key()) {
-    case Key_Plus:
-	actionExpandRow(m_curRow);
+    case Qt::Key_Plus:
+	actionExpandRow(textCursor().blockNumber());
 	return;
-    case Key_Minus:
-	actionCollapseRow(m_curRow);
+    case Qt::Key_Minus:
+	actionCollapseRow(textCursor().blockNumber());
 	return;
-    case Key_Up:
-	if (m_curRow > 0) {
-	    setCursorPosition(m_curRow-1, 0);
-	}
+    case Qt::Key_Up:
+	moveCursor(QTextCursor::PreviousBlock);
 	return;
-    case Key_Down:
-	if (m_curRow < paragraphs()-1) {
-	    setCursorPosition(m_curRow+1, 0);
-	}
+    case Qt::Key_Down:
+	moveCursor(QTextCursor::NextBlock);
 	return;
-    case Key_Home:
-	setCursorPosition(0, 0);
+    case Qt::Key_Home:
+	moveCursor(QTextCursor::Start);
 	return;
-    case Key_End:
-	setCursorPosition(paragraphs()-1, 0);
+    case Qt::Key_End:
+	moveCursor(QTextCursor::End);
 	return;
-    case Key_Next:
-    case Key_Prior:
-	top = viewportToContents(QPoint(0,0));
-	top1 = paragraphAt(top);
+    case Qt::Key_PageUp:
+    case Qt::Key_PageDown:
+	top1 = firstVisibleBlock().blockNumber();
     }
 
-    QTextEdit::keyPressEvent(ev);
+    QPlainTextEdit::keyPressEvent(ev);
 
     switch (ev->key()) {
-    case Key_Next:
-    case Key_Prior:
-	top = viewportToContents(QPoint(0,0));
-	top2 = paragraphAt(top);
-	setCursorPosition(m_curRow+(top2-top1), 0);
+    case Qt::Key_PageUp:
+    case Qt::Key_PageDown:
+	top2 = firstVisibleBlock().blockNumber();
+	{
+	    QTextCursor cursor = textCursor();
+	    if (top1 < top2)
+		cursor.movePosition(QTextCursor::NextBlock,
+				    QTextCursor::MoveAnchor, top2-top1);
+	    else
+		cursor.movePosition(QTextCursor::PreviousBlock,
+				    QTextCursor::MoveAnchor, top1-top2);
+	    setTextCursor(cursor);
+	}
     }
 }
 
-static inline bool isident(QChar c)
-{
-    return c.isLetterOrNumber() || c.latin1() == '_';
-}
-
 bool SourceWindow::wordAtPoint(const QPoint& p, QString& word, QRect& r)
 {
-    QPoint pv = viewportToContents(p - viewport()->pos());
-    int row, col  = charAt(pv, &row);
-    if (row < 0 || col < 0)
+    QTextCursor cursor = cursorForPosition(viewport()->mapFrom(this, p));
+    if (cursor.isNull())
 	return false;
 
-    // isolate the word at row, col
-    QString line = text(row);
-    if (!isident(line[col]))
+    cursor.select(QTextCursor::WordUnderCursor);
+    word = cursor.selectedText();
+
+    if (word.isEmpty())
 	return false;
 
-    int begin = col;
-    while (begin > 0 && isident(line[begin-1]))
-	--begin;
-    do
-	++col;
-    while (col < int(line.length()) && isident(line[col]));
+    // keep only letters and digits
+    QRegExp w("[\\dA-Za-z_]+");
+    if (w.indexIn(word) < 0)
+	return false;
+    word = w.cap();
 
     r = QRect(p, p);
-    r.addCoords(-5,-5,5,5);
-    word = line.mid(begin, col-begin);
+    r.adjust(-5,-5,5,5);
     return true;
 }
 
 void SourceWindow::paletteChange(const QPalette& oldPal)
 {
     setFont(KGlobalSettings::fixedFont());
-    QTextEdit::paletteChange(oldPal);
+    QPlainTextEdit::paletteChange(oldPal);
 }
 
 /*
@@ -498,7 +491,7 @@ void SourceWindow::disassembled(int lineNo, const std::list<DisassembledCode>& d
 	expandRow(row);
     } else {
 	// clear expansion marker
-	update();
+	m_lineInfoArea->update();
     }
 }
 
@@ -602,23 +595,18 @@ void SourceWindow::expandRow(int row)
     // remove PC (must be set again in slot of signal expanded())
     m_lineItems[row] &= ~(liPC|liPCup);
 
-    // adjust current row
-    if (m_curRow > row) {
-	m_curRow += disass.size();
-	// highlight is moved automatically
-    }
-
     // insert new lines
     setUpdatesEnabled(false);
     ++row;
+    m_rowToLine.insert(m_rowToLine.begin()+row, disass.size(), line);
+    m_lineItems.insert(m_lineItems.begin()+row, disass.size(), 0);
+
+    QTextCursor cursor(document()->findBlockByNumber(row));
     for (size_t i = 0; i < disass.size(); i++) {
-	m_rowToLine.insert(m_rowToLine.begin()+row, line);
-	m_lineItems.insert(m_lineItems.begin()+row, 0);
-	insertParagraph(disass[i], row++);
+	cursor.insertText(disass[i]);
+	cursor.insertBlock();
     }
     setUpdatesEnabled(true);
-    viewport()->update();
-    update();		// line items
 
     emit expanded(line);		/* must set PC */
 }
@@ -634,28 +622,22 @@ void SourceWindow::collapseRow(int row)
 	end++;
     }
     ++row;
-    // adjust current row
-    if (m_curRow >= row) {
-	m_curRow -= end-row;
-	if (m_curRow < row)	// was m_curRow in disassembled code?
-	    m_curRow = -1;
-    }
     setUpdatesEnabled(false);
+    QTextCursor cursor(document()->findBlockByNumber(end-1));
     while (--end >= row) {
 	m_rowToLine.erase(m_rowToLine.begin()+end);
 	m_lineItems.erase(m_lineItems.begin()+end);
-	removeParagraph(end);
+	cursor.select(QTextCursor::BlockUnderCursor);
+	cursor.removeSelectedText();
     }
     setUpdatesEnabled(true);
-    viewport()->update();
-    update();		// line items
 
     emit collapsed(line);
 }
 
 void SourceWindow::activeLine(int& line, DbgAddr& address)
 {
-    int row = m_curRow;
+    int row = textCursor().blockNumber();
 
     int sourceRow;
     line = rowToLine(row, &sourceRow);
@@ -723,29 +705,28 @@ void SourceWindow::setTabWidth(int numChars)
 {
     if (numChars <= 0)
 	numChars = 8;
-    QFontMetrics fm(currentFont());
+    QFontMetrics fm(document()->defaultFont());
     QString s;
     int w = fm.width(s.fill('x', numChars));
     setTabStopWidth(w);
 }
 
-void SourceWindow::cursorChanged(int row)
+void SourceWindow::cursorChanged()
 {
-    if (row == m_curRow)
-	return;
+    QList<QTextEdit::ExtraSelection> extraSelections;
+    QTextEdit::ExtraSelection selection;
 
-    if (m_curRow >= 0 && m_curRow < paragraphs())
-	clearParagraphBackground(m_curRow);
-    m_curRow = row;
-    setParagraphBackgroundColor(row, colorGroup().background());
+    selection.format.setBackground(QColor("#E7E7E7"));
+    selection.format.setProperty(QTextFormat::FullWidthSelection, true);
+    selection.cursor = textCursor();
+    selection.cursor.clearSelection();
+    extraSelections.append(selection);
+    setExtraSelections(extraSelections);
 }
 
 /*
- * We must override the context menu handling because QTextEdit's handling
- * requires that it receives ownership of the popup menu; but the popup menu
- * returned from the GUI factory is owned by the factory.
+ * Show our own context menu.
  */
-
 void SourceWindow::contextMenuEvent(QContextMenuEvent* e)
 {
     // get the context menu from the GUI factory
@@ -753,24 +734,30 @@ void SourceWindow::contextMenuEvent(QContextMenuEvent* e)
     do
 	top = top->parentWidget();
     while (!top->isTopLevel());
-    KMainWindow* mw = static_cast<KMainWindow*>(top);
-    QPopupMenu* m =
-	static_cast<QPopupMenu*>(mw->factory()->container("popup_files", mw));
-    m->exec(e->globalPos());
+    KXmlGuiWindow* mw = static_cast<KXmlGuiWindow*>(top);
+    QMenu* m = static_cast<QMenu*>(mw->factory()->container("popup_files", mw));
+    if (m)
+	m->exec(e->globalPos());
 }
 
-bool SourceWindow::eventFilter(QObject* watched, QEvent* e)
+void LineInfoArea::paintEvent(QPaintEvent* e)
 {
-    if (e->type() == QEvent::ContextMenu && watched == viewport())
-    {
-	contextMenuEvent(static_cast<QContextMenuEvent*>(e));
-	return true;
-    }
-    return QTextEdit::eventFilter(watched, e);
+    QPainter p(this);
+    static_cast<SourceWindow*>(parent())->drawLineInfoArea(&p, e);
+}
+
+void LineInfoArea::mousePressEvent(QMouseEvent* ev)
+{
+    static_cast<SourceWindow*>(parent())->infoMousePress(ev);
+}
+
+void LineInfoArea::contextMenuEvent(QContextMenuEvent* e)
+{
+    static_cast<SourceWindow*>(parent())->contextMenuEvent(e);
 }
 
 HighlightCpp::HighlightCpp(SourceWindow* srcWnd) :
-	QSyntaxHighlighter(srcWnd),
+	QSyntaxHighlighter(srcWnd->document()),
 	m_srcWnd(srcWnd)
 {
 }
@@ -860,13 +847,19 @@ static const QString ckw[] =
     "xor_eq"
 };
 
-int HighlightCpp::highlightParagraph(const QString& text, int state)
+void HighlightCpp::highlightBlock(const QString& text)
+{
+    int state = previousBlockState();
+    state = highlight(text, state);
+    setCurrentBlockState(state);
+}
+
+int HighlightCpp::highlight(const QString& text, int state)
 {
-    int row = currentParagraph();
     // highlight assembly lines
-    if (m_srcWnd->isRowDisassCode(row))
+    if (m_srcWnd->isRowDisassCode(currentBlock().blockNumber()))
     {
-	setFormat(0, text.length(), blue);
+	setFormat(0, text.length(), Qt::blue);
 	return state;
     }
 
@@ -874,17 +867,17 @@ int HighlightCpp::highlightParagraph(const QString& text, int state)
 	state = 0;
 
     // check for preprocessor line
-    if (state == 0 && text.stripWhiteSpace().startsWith("#"))
+    if (state == 0 && text.trimmed().startsWith("#"))
     {
-	setFormat(0, text.length(), QColor("dark green"));
+	setFormat(0, text.length(), QColor("darkgreen"));
 	return 0;
     }
 
     // a font for keywords
-    QFont identFont = textEdit()->currentFont();
-    identFont.setBold(!identFont.bold());
+    QTextCharFormat identFont;
+    identFont.setFontWeight(QFont::Bold);
 
-    unsigned start = 0;
+    int start = 0;
     while (start < text.length())
     {
 	int end;
@@ -892,15 +885,15 @@ int HighlightCpp::highlightParagraph(const QString& text, int state)
 	case hlCommentLine:
 	    end = text.length();
 	    state = 0;
-	    setFormat(start, end-start, QColor("gray50"));
+	    setFormat(start, end-start, QColor("gray"));
 	    break;
 	case hlCommentBlock:
-	    end = text.find("*/", start);
+	    end = text.indexOf("*/", start);
 	    if (end >= 0)
 		end += 2, state = 0;
 	    else
 		end = text.length();
-	    setFormat(start, end-start, QColor("gray50"));
+	    setFormat(start, end-start, QColor("gray"));
 	    break;
 	case hlString:
 	    for (end = start+1; end < int(text.length()); end++) {
@@ -913,7 +906,7 @@ int HighlightCpp::highlightParagraph(const QString& text, int state)
 		}
 	    }
 	    state = 0;
-	    setFormat(start, end-start, QColor("dark red"));
+	    setFormat(start, end-start, QColor("darkred"));
 	    break;
 	case hlIdent:
 	    for (end = start+1; end < int(text.length()); end++) {
@@ -926,7 +919,7 @@ int HighlightCpp::highlightParagraph(const QString& text, int state)
 	    {
 		setFormat(start, end-start, identFont);
 	    } else {
-		setFormat(start, end-start, m_srcWnd->colorGroup().text());
+		setFormat(start, end-start, m_srcWnd->palette().color(QPalette::WindowText));
 	    }
 	    break;
 	default:
@@ -949,15 +942,15 @@ int HighlightCpp::highlightParagraph(const QString& text, int state)
 		    state = hlString;
 		    break;
 		}
-		else if (text[end] >= 'A' && text[end] <= 'Z' ||
-			 text[end] >= 'a' && text[end] <= 'z' ||
+		else if ((text[end] >= 'A' && text[end] <= 'Z') ||
+			 (text[end] >= 'a' && text[end] <= 'z') ||
 			 text[end] == '_')
 		{
 		    state = hlIdent;
 		    break;
 		}
 	    }
-	    setFormat(start, end-start, m_srcWnd->colorGroup().text());
+	    setFormat(start, end-start, m_srcWnd->palette().color(QPalette::WindowText));
 	}
 	start = end;
     }
