@@ -13,13 +13,13 @@ int main()
    const wchar_t* nullPtr = 0;
    const wchar_t* uninitializedPtr = (const wchar_t*)0xdeadbeef;
    const wchar_t* str = L"abc";
-   wchar_t* str2 = L"def";
+   const wchar_t* str2 = L"def";
    const char* shortStr = "12345";
 
-   WChar s = { 0, L"Some string" };
-   s.cwstr = s.wstr;
+   wchar_t wstr[64] = L"Some string", *wstrPtr = wstr;
 
-   wchar_t wstr[64], *wstrPtr = wstr;
+   WChar s = { 0, wstr };
+   s.cwstr = s.wstr;
 
    wprintf(L"wide string: %S\n", str);
 
